/*
 * Copyright (c) 2013-2019 Thomas Isaac Lightburn
 *
 *
 * This file is part of OpenKJ.
 *
 * OpenKJ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SCROLLTEXT_H
#define SCROLLTEXT_H

#include <QStaticText>
#include <QTimer>

#ifdef USE_GL
#include <QGLWidget>
#else
#include <QWidget>
#endif

#ifdef USE_GL
class ScrollText : public QGLWidget
#else
class ScrollText : public QWidget
#endif
{
    Q_OBJECT
    Q_PROPERTY(QString text READ text WRITE setText)
    Q_PROPERTY(QString separator READ separator WRITE setSeparator)

private:
    void updateText();
    QString _text;
    QString _separator;
    QStaticText staticText;
    int singleTextWidth;
    QSize wholeTextSize;
    int leftMargin;
    bool scrollEnabled;
    int scrollPos;
    QImage alphaChannel;
    QImage buffer;
    QTimer timer;
    bool enabled;

public:
    explicit ScrollText(QWidget *parent = 0);

protected:
    virtual void paintEvent(QPaintEvent *);
    virtual void resizeEvent(QResizeEvent *);

private slots:
    virtual void timer_timeout();

public slots:
    QString text() const;
    void setText(QString text);
    QString separator() const;
    void setSeparator(QString separator);
    void refresh();
    void setSpeed(int speed);
    void enable(bool Enabled);

};

#endif // SCROLLTEXT_H
